unit MCSkin;
{$ifdef FPC}{$mode objfpc}{$h+}{$endif}
interface
uses
 mclasses,mseglob,mseapplication,mseclasses,msedatamodules,mseskin,msegui,
 msegraphutils,msegraphics,MCMethodListU,msebitmap;

type
 TMCColors = (clBackground, clForeground, clText, clSelected, clBannerTop, clBannerBottom,
              clBannerText, clBannerIcon, clIcon, clButtonFrame,
              clButtonFrameMouse, clButtonFrameClick, clButtonFrameActive,
              clButtonFrameDisabled, clButtonFaceTop, clButtonFaceMiddle,
              clButtonFaceBottom, clButtonFaceTopA, clButtonFaceMiddleA,
              clButtonFaceBottomA, clControlFrame, clScrollbar, clScrollbarA, 
              clScrollbarBackground, clTrackBarFill, clTrackBarEmpty, clGridLine);
 TMCColorsArray = array[TMCColors] of colorty;
 TMCSkinEvents = (mseColorChanged);
 TOnMCColorChanged = procedure(const Sender: TObject; AType: TMCColors; AColor: colorty) of object;

 { TMCSkin }

 TMCSkin = class(tmsedatamodule)
   scMain: tskincontroller;
   buttonFrame: tframecomp;
   editFrame: tframecomp;
   fontBanner: tfontcomp;
   bannerFace: tfacecomp;
   buttonFace: tfacecomp;
   sliderHFace1: tfacecomp;
   sliderButtonFrame: tframecomp;
   sliderVFace1: tfacecomp;
   sliderVFace2: tfacecomp;
   sliderHFace2: tfacecomp;
   menuitemFrameA: tframecomp;
   menuitemFrame: tframecomp;
   menuitemFaceA: tfacecomp;
   buttonframeList: tfacelist;
   buttonframeImgLst: timagelist;
   buttongradientFace: tfacecomp;
   buttongradientFaceA: tfacecomp;
   buttonframeImgLstA: timagelist;
   scFlat: tskincontroller;
   buttonFrameFlat: tframecomp;
   splitterFrame: tframecomp;
   scrollfacelistV: tfacelist;
   scrollframeImgLst: timagelist;
   scrollFrameV: tframecomp;
   scrollFaceV: tfacecomp;
   scrollFaceVA: tfacecomp;
   scrollframeImgLstA: timagelist;
   procedure MainSkinActivated(const sender: TObject);
 private
   FEvents: array[TMCSkinEvents] of TMCMethodList;
   FUpdateCount: Byte;
   procedure CallColorChanged(AType: TMCColors; AColor: colorty);
   procedure AddHandler(AHandler: TMethod; AKind: TMCSkinEvents; AsLast: Boolean);
   procedure RemoveHandler(AHandler: TMethod; AKind: TMCSkinEvents);
 protected
   procedure doafterload; override;
 public
   constructor create(aowner: tcomponent); overload; override;
   destructor destroy; override;
   function GetMCColor(AType: TMCColors): colorty;
   procedure SetMCColor(AType: TMCColors; AColor: colorty); overload;
   procedure SetMCColor(constref AArray: TMCColorsArray); overload;

   function GetDarkTheme: TMCColorsArray;
   function GetLightTheme: TMCColorsArray;

   procedure BeginUpdateColors;
   procedure EndUpdateColors;

   // Events
   procedure AddHandlerColorChanged(AEvent: TOnMCColorChanged; AsLast: Boolean = True);
   procedure RemoveHandlerColorChanged(AEvent: TOnMCColorChanged);
 end;

var
 MCSkinMO: TMCSkin;
 
implementation

uses
 mcskin_mfm, sysutils;
 
function GetRealColor(AColor: colorty): colorty; inline;
begin
  Result := colorty(colortorgb(AColor));
end;

{ TMCSkin }

procedure TMCSkin.CallColorChanged(AType: TMCColors; AColor: colorty);
var i: Integer;
begin
  if FEvents[mseColorChanged]<>nil then
    for i:=0 to FEvents[mseColorChanged].Count-1 do
      TOnMCColorChanged(FEvents[mseColorChanged][i]^)(Self, AType, AColor);
end;

procedure TMCSkin.AddHandler(AHandler: TMethod; AKind: TMCSkinEvents;
  AsLast: Boolean);
begin
  if FEvents[AKind]=nil then
    FEvents[AKind] := TMCMethodList.Create;
  FEvents[AKind].Add(AHandler);
end;

procedure TMCSkin.RemoveHandler(AHandler: TMethod; AKind: TMCSkinEvents);
begin
  if FEvents[AKind]<>nil then
    FEvents[AKind].Remove(AHandler);
end;

procedure TMCSkin.doafterload;
begin
  inherited doafterload;
end;

constructor TMCSkin.create(aowner: tcomponent);
begin
  inherited create(aowner);
  //DefLightIconColor := rgbtocolor(254, 254, 254);
  //DefIconColor := rgbtocolor(133, 32, 14);
  FUpdateCount := 0;
  if scMain.active then
    SetMCColor(GetLightTheme);
end;

destructor TMCSkin.destroy;
var e: TMCSkinEvents;
begin
  for e:=Low(TMCSkinEvents) to High(TMCSkinEvents) do
    FreeAndNil(FEvents[e]);
  inherited destroy;
end;

function TMCSkin.GetMCColor(AType: TMCColors): colorty;
begin
  case AType of
  clBackground: Result := GetRealColor(cl_background);
  clForeground: Result := GetRealColor(cl_foreground);
  clText: Result := GetRealColor(cl_text);
  clSelected: Result := GetRealColor(cl_gridselect);
  clBannerTop: Result := GetRealColor(bannerFace.template.fade_color.items[0]);
  clBannerBottom: Result := GetRealColor(bannerFace.template.fade_color.items[1]);
  clBannerText: Result := GetRealColor(fontBanner.template.color);
  clBannerIcon: Result := GetRealColor(cl_user0);
  clButtonFrame: Result := GetRealColor(buttonFrame.template.colorframe);
  clButtonFrameActive: Result := GetRealColor(buttonFrame.template.colorframeactive);
  clButtonFrameClick: Result := GetRealColor(buttonFrame.template.colorframeclicked);
  clButtonFrameMouse: Result := GetRealColor(buttonFrame.template.colorframemouse);
  clButtonFrameDisabled: Result := GetRealColor(buttonFrame.template.colorframedisabled);
  clButtonFaceTop: Result := GetRealColor(buttongradientFace.template.fade_color.items[0]);
  clButtonFaceMiddle: Result := GetRealColor(buttongradientFace.template.fade_color.items[1]);
  clButtonFaceBottom: Result := GetRealColor(buttongradientFace.template.fade_color.items[2]);
  clButtonFaceTopA: Result := GetRealColor(buttongradientFaceA.template.fade_color.items[0]);
  clButtonFaceMiddleA: Result := GetRealColor(buttongradientFaceA.template.fade_color.items[1]);
  clButtonFaceBottomA: Result := GetRealColor(buttongradientFaceA.template.fade_color.items[2]);
  clIcon: Result := GetRealColor(cl_user1);
  clControlFrame: Result := GetRealColor(editFrame.template.colorframe);
  clScrollbarBackground: Result := GetRealColor(cl_scrollbarpattern);
  clTrackBarFill: Result := sliderHFace1.template.fade_color.items[0];
  clTrackBarEmpty: Result := sliderVFace2.template.fade_color.items[0];
  clGridLine: Result := GetRealColor(cl_gridline);
  clScrollbar: Result := scrollFaceV.template.fade_color.items[0];
  clScrollbarA: Result := scrollFaceVA.template.fade_color.items[0];
  else
    Result := GetRealColor(cl_default);
  end;
end;

procedure TMCSkin.SetMCColor(AType: TMCColors; AColor: colorty);
begin  
  case AType of
  clBackground: setcolormapvalue(cl_background, AColor);
  clForeground: setcolormapvalue(cl_foreground, AColor);
  clText: setcolormapvalue(cl_text, AColor);
  clSelected: 
  begin
    setcolormapvalue(cl_gridselect, AColor);
    setcolormapvalue(cl_selectedtextbackground, AColor);
  end;
  clBannerTop: bannerFace.template.fade_color.items[0] := AColor;
  clBannerBottom: bannerFace.template.fade_color.items[1] := AColor;
  clBannerText: fontBanner.template.color := AColor;
  clBannerIcon: setcolormapvalue(cl_user0, AColor);
  clButtonFrame:
    begin
      buttonFrame.template.colorframe := AColor;
      buttonframeImgLst.bitmap.init(AColor);
      setcolormapvalue(cl_defaultrect, AColor);
      setcolormapvalue(cl_gridlinefix, AColor);
      scMain.splitter_colorgrip := AColor;
    end;
  clButtonFrameActive: 
    begin
      buttonFrame.template.colorframeactive := AColor;
      buttonframeImgLstA.bitmap.init(AColor);
    end;
  clButtonFrameClick: buttonFrame.template.colorframeclicked := AColor;
  clButtonFrameMouse:
    begin
      buttonFrame.template.colorframemouse := AColor;
      //menuitemFrame.template.colorframemouse := AColor;
    end;
  clButtonFrameDisabled: buttonFrame.template.colorframedisabled := AColor;
  clButtonFaceTop: buttongradientFace.template.fade_color.items[0] := AColor;
  clButtonFaceMiddle: buttongradientFace.template.fade_color.items[1] := AColor;
  clButtonFaceBottom: buttongradientFace.template.fade_color.items[2] := AColor;
  clButtonFaceTopA: buttongradientFaceA.template.fade_color.items[0] := AColor;
  clButtonFaceMiddleA: buttongradientFaceA.template.fade_color.items[1] := AColor;
  clButtonFaceBottomA: buttongradientFaceA.template.fade_color.items[2] := AColor;
  clIcon: setcolormapvalue(cl_user1, AColor);
  clControlFrame: editFrame.template.colorframe := AColor;
  clScrollbarBackground: setcolormapvalue(cl_scrollbarpattern, AColor);
  clTrackBarFill:
    begin
      sliderHFace1.template.fade_color.items[0] := AColor;
      sliderVFace1.template.fade_color.items[0] := AColor;
    end;
  clTrackBarEmpty:
    begin
      sliderHFace2.template.fade_color.items[0] := AColor;
      sliderVFace2.template.fade_color.items[0] := AColor;
    end;
  clGridLine: setcolormapvalue(cl_gridline, AColor);
  clScrollbar: 
    begin
      scrollFaceV.template.fade_color.items[0] := AColor;
      scrollframeImgLst.bitmap.init(AColor);
    end;
  clScrollbarA: 
    begin
      scrollFaceVA.template.fade_color.items[0] := AColor;
      scrollframeImgLstA.bitmap.init(AColor);
    end;
  end;
  //setcolormapvalue(cl_grayedshadow, cl_transparent);
  CallColorChanged(AType,AColor);
  if FUpdateCount=0 then
    application.invalidate;
end;

procedure TMCSkin.SetMCColor(constref AArray: TMCColorsArray);
var i: TMCColors;
begin
  BeginUpdateColors;
  try
    for i:=Low(AArray) to High(AArray) do
      SetMCColor(i, AArray[i]);
  finally
    EndUpdateColors;
  end;
end;

function TMCSkin.GetDarkTheme: TMCColorsArray;
//var i: TMCColors;

begin
  //for i:=Low(Result) to High(Result) do
  //  Result[i] := cl_red;

  FillDWord(Result, Length(Result), GetRealColor(cl_transparent));
  Result[clBackground] := $2E2E2E;
  Result[clForeground] := $262626;
  Result[clText]       := $D7D7D7;
  Result[clBannerText] := $D7D7D7;
  Result[clIcon]       := $BDBDBD;
  Result[clBannerIcon] := $BDBDBD;
  Result[clBannerTop] := Result[clBackground];
  Result[clBannerBottom] := Result[clBackground];
  Result[clButtonFrame] := $404040;
  //Result[clButtonFrame] := cl_red;
  Result[clButtonFrameMouse] := $6d6c6c;
  Result[clButtonFrameClick] := $4f4f4f;
  Result[clButtonFrameDisabled] := Result[clButtonFrame];
  Result[clButtonFrameActive] := GetRealColor(cl_black);
  Result[clButtonFaceTop] := Result[clBackground];
  Result[clButtonFaceMiddle] := Result[clBackground];
  Result[clButtonFaceBottom] := Result[clBackground];
  Result[clControlFrame] := Result[clButtonFrame];
  Result[clSelected] := $161616;
  Result[clTrackBarFill] := Result[clSelected];
  Result[clTrackBarEmpty] := Result[clIcon];
  //writeln(hexStr(LongInt(Result[clIcon]),100));
end;

function TMCSkin.GetLightTheme: TMCColorsArray;
begin
  FillDWord(Result, Length(Result), GetRealColor(cl_transparent));
  {Result[clBackground] := $EFF0F1;
  Result[clForeground] := $FCFCFC;
  Result[clText]       := $61180c;
  Result[clBannerText] := $D7D7D7;
  Result[clIcon]       := $800000;
  Result[clBannerIcon] := $FAFAFA;
  Result[clBannerTop] := $C03518;
  Result[clBannerBottom] := $671B0D;
  Result[clButtonFrame] := $61180c;
  Result[clButtonFrameMouse] := $a62a14;
  Result[clButtonFrameClick] := $4f4f4f;
  Result[clButtonFrameDisabled] := Result[clButtonFrame];
  Result[clButtonFrameActive] := GetRealColor(cl_black);
  Result[clButtonFaceTop] := Result[clBackground];
  Result[clButtonFaceMiddle] := Result[clBackground];
  Result[clButtonFaceBottom] := Result[clBackground];
  Result[clControlFrame] := Result[clButtonFrame];
  Result[clSelected] := $ff4520;
  Result[clTrackBarFill] := Result[clSelected];
  Result[clTrackBarEmpty] := Result[clIcon];}
  
  Result[clBackground] := $EFF0F1;
  Result[clForeground] := $FCFCFC;
  Result[clText]       := $39274a;
  Result[clBannerText] := $ebfff6;
  Result[clIcon]       := $53396b;
  Result[clBannerIcon] := $FAFAFA;
  Result[clBannerTop] := $b986cb;
  Result[clBannerBottom] := $613f91;
  Result[clButtonFrame] := $755197;
  Result[clButtonFrameMouse] := $9664e6;
  Result[clButtonFrameClick] := $422b64;
  Result[clButtonFrameDisabled] := Result[clButtonFrame];
  Result[clButtonFrameActive] := $563b6f;
  Result[clButtonFaceTop] := $825aa7;
  Result[clButtonFaceMiddle] := $825aa7;
  Result[clButtonFaceBottom] := $825aa7;
  Result[clButtonFaceTopA] := $684886;
  Result[clButtonFaceMiddleA] := $684886;
  Result[clButtonFaceBottomA] := $684886;
  Result[clControlFrame] := $c79def;
  Result[clSelected] := $9a6ac6;
  Result[clTrackBarFill] := Result[clSelected];
  Result[clTrackBarEmpty] := Result[clIcon];  
  Result[clGridLine] := $faeaff;
  Result[clScrollbar] := cl_gray;
  Result[clScrollbarA] := cl_dkgray;
end;

procedure TMCSkin.BeginUpdateColors;
begin
  FUpdateCount += 1;
end;

procedure TMCSkin.EndUpdateColors;
begin
  if FUpdateCount>0 then
  begin
    FUpdateCount -= 1;
    if FUpdateCount=0 then
      application.invalidate;
  end;
end;

procedure TMCSkin.AddHandlerColorChanged(AEvent: TOnMCColorChanged;
  AsLast: Boolean);
begin
  AddHandler(TMethod(AEvent), mseColorChanged, AsLast);
end;

procedure TMCSkin.RemoveHandlerColorChanged(AEvent: TOnMCColorChanged);
begin
  RemoveHandler(TMethod(AEvent), mseColorChanged);
end;

procedure TMCSkin.MainSkinActivated(const sender: TObject);
begin
  scFlat.Active := scMain.Active;
end;
 
initialization
  application.createdatamodule(TMCSkin, MCSkinMO);
end.
